@@ -4,27 +4,26 @@
 
   Author: Pekka Riikonen <priikone@silcnet.org>
 
-  Copyright (C) 1998 - 2001 Pekka Riikonen
+  Copyright (C) 1998 - 2004 Pekka Riikonen
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
-  the Free Software Foundation; either version 2 of the License, or
-  (at your option) any later version.
-  
+  the Free Software Foundation; version 2 of the License.
+
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
 
 */
-/* $Id: silcunixschedule.c,v 1.11 2002/09/16 10:20:22 priikone Exp $ */
+/* $Id: silcunixschedule.c,v 1.11.2.3 2005/05/11 16:54:14 priikone Exp $ */
 
 #include "silcincludes.h"
 #include "silcschedule_i.h"
 
 /* Calls normal select() system call. */
 
-int silc_select(SilcScheduleFd fds, SilcUInt32 fds_count, 
+int silc_select(SilcScheduleFd fds, SilcUInt32 fds_count,
 		struct timeval *timeout)
 {
   fd_set in, out;
@@ -37,6 +36,11 @@ int silc_select(SilcScheduleFd fds, SilcUInt32 fds_count,
     if (!fds[i].events)
       continue;
 
+#ifdef FD_SETSIZE
+    if (fds[i].fd >= FD_SETSIZE)
+      break;
+#endif /* FD_SETSIZE */
+
     if (fds[i].fd > max_fd)
       max_fd = fds[i].fd;
 
@@ -56,6 +60,11 @@ int silc_select(SilcScheduleFd fds, SilcUInt32 fds_count,
     if (!fds[i].events)
       continue;
 
+#ifdef FD_SETSIZE
+    if (fds[i].fd >= FD_SETSIZE)
+      break;
+#endif /* FD_SETSIZE */
+
     if (FD_ISSET(fds[i].fd, &in))
       fds[i].revents |= SILC_TASK_READ;
     if (FD_ISSET(fds[i].fd, &out))
@@ -119,10 +128,10 @@ void *silc_schedule_internal_init(SilcSchedule schedule,
     return NULL;
   }
 
-  internal->wakeup_task = 
+  internal->wakeup_task =
     silc_schedule_task_add(schedule, internal->wakeup_pipe[0],
 			   silc_schedule_wakeup_cb, internal,
-			   0, 0, SILC_TASK_FD, 
+			   0, 0, SILC_TASK_FD,
 			   SILC_TASK_PRI_NORMAL);
   if (!internal->wakeup_task) {
     SILC_LOG_ERROR(("Could not add a wakeup task, threads won't work"));
@@ -138,6 +147,9 @@ void *silc_schedule_internal_init(SilcSchedule schedule,
   return (void *)internal;
 }
 
+void silc_schedule_internal_signals_block(void *context);
+void silc_schedule_internal_signals_unblock(void *context);
+
 /* Uninitializes the platform specific scheduler context. */
 
 void silc_schedule_internal_uninit(void *context)
@@ -180,7 +192,9 @@ void silc_schedule_internal_signal_register(void *context,
   if (!internal)
     return;
 
-  sigprocmask(SIG_BLOCK, &internal->signals, &internal->signals_blocked);
+  SILC_LOG_DEBUG(("Registering signal %d", signal));
+
+  silc_schedule_internal_signals_block(context);
 
   for (i = 0; i < SIGNAL_COUNT; i++) {
     if (!internal->signal_call[i].signal) {
@@ -192,7 +206,7 @@ void silc_schedule_internal_signal_register(void *context,
     }
   }
 
-  sigprocmask(SIG_SETMASK, &internal->signals_blocked, NULL);
+  silc_schedule_internal_signals_unblock(context);
   sigaddset(&internal->signals, signal);
 }
 
@@ -207,7 +221,9 @@ void silc_schedule_internal_signal_unregister(void *context,
   if (!internal)
     return;
 
-  sigprocmask(SIG_BLOCK, &internal->signals, &internal->signals_blocked);
+  SILC_LOG_DEBUG(("Unregistering signal %d", signal));
+
+  silc_schedule_internal_signals_block(context);
 
   for (i = 0; i < SIGNAL_COUNT; i++) {
     if (internal->signal_call[i].signal == signal &&
@@ -220,7 +236,7 @@ void silc_schedule_internal_signal_unregister(void *context,
     }
   }
 
-  sigprocmask(SIG_SETMASK, &internal->signals_blocked, NULL);
+  silc_schedule_internal_signals_unblock(context);
   sigdelset(&internal->signals, signal);
 }
 
@@ -234,16 +250,19 @@ void silc_schedule_internal_signal_call(void *context, SilcUInt32 signal)
   if (!internal)
     return;
 
-  sigprocmask(SIG_BLOCK, &internal->signals, &internal->signals_blocked);
+  silc_schedule_internal_signals_block(context);
 
   for (i = 0; i < SIGNAL_COUNT; i++) {
-    if (internal->signal_call[i].signal == signal)
+    if (internal->signal_call[i].signal == signal) {
       internal->signal_call[i].call = TRUE;
+      SILC_LOG_DEBUG(("Scheduling signal %d to be called",
+		      internal->signal_call[i].signal));
+    }
   }
 
-  sigprocmask(SIG_SETMASK, &internal->signals_blocked, NULL);
+  silc_schedule_internal_signals_unblock(context);
 }
-                                        
+
 /* Call all signals */
 
 void silc_schedule_internal_signals_call(void *context,
@@ -252,14 +271,18 @@ void silc_schedule_internal_signals_call(void *context,
   SilcUnixScheduler internal = (SilcUnixScheduler)context;
   int i;
 
+  SILC_LOG_DEBUG(("Start"));
+
   if (!internal)
     return;
 
-  sigprocmask(SIG_BLOCK, &internal->signals, &internal->signals_blocked);
+  silc_schedule_internal_signals_block(context);
 
   for (i = 0; i < SIGNAL_COUNT; i++) {
     if (internal->signal_call[i].call &&
         internal->signal_call[i].callback) {
+      SILC_LOG_DEBUG(("Calling signal %d callback",
+		      internal->signal_call[i].signal));
       internal->signal_call[i].callback(schedule, internal->app_context,
 					SILC_TASK_INTERRUPT,
 					internal->signal_call[i].signal,
@@ -268,7 +291,7 @@ void silc_schedule_internal_signals_call(void *context,
     }
   }
 
-  sigprocmask(SIG_SETMASK, &internal->signals_blocked, NULL);
+  silc_schedule_internal_signals_unblock(context);
 }
 
 /* Block registered signals in scheduler. */
